function [y,info_bits] = Array_Tx_PulseSechant(PeakPower,FWHM,ExcessPulseSlots,CenterPosition,...
                         PreSpaces,PostSpaces,LaserNumber,LaserSpacing)
%
% Usage: [y,info_bits] = Array_Tx_PulseSechant(PeakPower,FWHM,ExcessPulseSlots,CenterPosition,...
%                        PreSpaces,PostSpaces,LaserNumber,LaserSpacing)
%
% Purpose = This module is to simulate an array of transmitters generating sechant optical pulses,
%           modulated by a bit sequence.
%          
% Array_Tx_PulseSechant = N_ch Array of Tx_PulseSechantOptical
%
% Input = Not required.
% Output1(y) = N channels of optical modulated signals separated by Laser spacing. (Size = [N x DataLength]) 
% Output2(info_bits) = Info_bits (Size = [N x Bits]) for BER calculation at the receiver.
%
%=== Physical Parameters with Default Values=======
% PeakPower ; [W] default = 1e-3;
% FWHM, Full width at half maximum power [sec], default = 1/3/BitRate;
% ExcessPulseSlots; Number of extra bit slots for each side to consider overlapping 
%                         with neighboring bits, default = 3;
% CenterPosition ; Relative position of the pulse peak wrt bit frame %[-0.5 0.5], default = 0 (center)
%
% PreSpaces = 2 (default); % number of leading zero bits, infobits_PRBS
% PostSpaces = 2 (default); % number of trailing zero bits, infobits_PRBS
%
% LaserNumber = ChannelNumber (default)
% LaserSpacing = ChannelSpacing (default)
%
%------------------------------------------------------------
% $ Revision: 1.1 $ Date: 2011/02/22 by Jong-Hyung Lee  
%------------------------------------------------------------
%
%====================================================================================
global t_range BitRate ChannelNumber ChannelSpacing

if nargin <8
    LaserSpacing = ChannelSpacing;
end
if nargin <7
    LaserNumber = ChannelNumber;
end
if nargin <6
    PostSpaces = 2;   
end    
if nargin<5,
    PreSpaces = 2;
end
if nargin<4,
   CenterPosition = 0;% 
end
if nargin<3,
   ExcessPulseSlots = 3;% 
end
if nargin<2,
   FWHM = 1/3/BitRate;% 
end
if nargin==0,
   PeakPower = 1e-3;% [W]
end
%==================================================================

if LaserNumber == 1
   
   [y,info_bits] = Tx_PulseSechantOptical(PeakPower,FWHM,ExcessPulseSlots,CenterPosition,...
                         PreSpaces,PostSpaces);
          
       
          
 elseif length(LaserSpacing) == 1   ;  % Equelly spaced case
   
   
	for k = 1:LaserNumber
   
       	 df_carrier(k,:) = exp(1i*2*pi*t_range*LaserSpacing*(k - (LaserNumber+1)/2));
         [x,info_bits(k,:)] = Tx_PulseSechantOptical(PeakPower,FWHM,ExcessPulseSlots,CenterPosition,...
                         PreSpaces,PostSpaces);
         y(k,:) = df_carrier(k,:).*x;
                 
	end
   
        
elseif length(LaserSpacing) > 1   ;  % Unequally spaced case
   
    for k = 1:LaserNumber ;
      	      
         df_carrier(k,:) = exp(1i*2*pi*t_range*LaserSpacing(k));
         [x,info_bits(k,:)] = Tx_PulseSechantOptical(PeakPower,FWHM,ExcessPulseSlots,CenterPosition,...
                         PreSpaces,PostSpaces);

         y(k,:) = df_carrier(k,:).*x;
           	   	       
	 end
   
end

