function y = E_FilterBSBessel(Elec_input,CenterFrequency,FilterOrder,BW3dB) 
%
% Usage: y = E_FilterBSBessel(Elec_input,CenterFrequency,FilterOrder,BW3dB)
%
% Purpose = This module is to filter an input electrical signal 
%          with an Bessel bandstop filter.
%
% Input Signal(Elec_input) = Electrical Signal 
% Output Signal(y) = Electrical Signal 
%
% === Physical Parameters === 
% CenterFrequency = Center frequency of the filter [Hz]
% FilterOrder = positive integer [1,100]
% BW3dB [Hz] = 3dB Bandwidth of band rejact(positive frequency)
%
% f_sym (global variable)
%
% Note) n_3dB is defined to normalize the 3dB bandwidth for FilterOrder >=3 
%               (at Cutoff Frequency, filter attenuation is greater than 3dB for FilterOrder >=3)
%
%----------------------------------------------------------
% $ Revision: 1.0 $ Date: 2003/05/07 by Jong-Hyung Lee 
% $ Revision: 1.1 $ Date: 2008/03/29 by Jong-Hyung Lee
%           ---- Unlike E_FilterLPBessel and E_FilterHPBessel, p1, p2 may not have zero.
%           ---- p1_center, p2_center finding has been modified.  
% $ Revision: 1.2 $ Date: 2011/02/22 by Jong-Hyung Lee
%------------------------------------------------------------
%
% Reference  1. Shammugan et al.,'Simulation of Communication Systems', Chapter 2, p69,  
%
%====================================================================================
global  f_sym 


if FilterOrder < 1 || FilterOrder > 100
  error('FilterOrder is out of range: ');


elseif FilterOrder == 1 || FilterOrder == 2;

	p1 = 1i*2*(f_sym-CenterFrequency)/BW3dB;p2 = 1i*2*(f_sym+CenterFrequency)/BW3dB;
	p1abs = abs(p1); p2abs = abs(p2);
    
    p1_center = find(p1abs == min(p1abs)); p2_center = find(p2abs == min(p2abs));
    
	B1(1,:) = 1+p1; B1(2,:) = 3 + 3*p1 + p1.*p1;
	B2(1,:) = 1+p2; B2(2,:) = 3 + 3*p2 + p2.*p2;

	T_bessel = sqrt(1-abs(B1(FilterOrder,p1_center(1))./B1(FilterOrder,:)+B2(FilterOrder,p2_center(1))./B2(FilterOrder,:)).^2);
  
else

	n_3dB = sqrt((2*FilterOrder-1)*log(2));
	%n_3dB will make 3dB bandwidth approximately equal to parameter value 'BW3dB' for FilterOrder >= 3
	p1 = 1i*2*(f_sym-CenterFrequency)/(BW3dB/n_3dB);p2 = 1i*2*(f_sym+CenterFrequency)/(BW3dB/n_3dB);
	p1abs = abs(p1); p2abs = abs(p2);
    
    p1_center = find(p1abs == min(p1abs)); p2_center = find(p2abs == min(p2abs));

	B1(1,:) = 1+p1; B1(2,:) = 3 + 3*p1 + p1.*p1;
	B2(1,:) = 1+p2; B2(2,:) = 3 + 3*p2 + p2.*p2;
	

	  for k = 3:FilterOrder
	
			B1(k,:) = (2*k-1)*B1(k-1,:) + p1.*p1.*B1(k-2,:);
			B2(k,:) = (2*k-1)*B2(k-1,:) + p2.*p2.*B2(k-2,:);

  	  end

	T_bessel = sqrt(1-abs(B1(FilterOrder,p1_center(1))./B1(FilterOrder,:)+B2(FilterOrder,p2_center(1))./B2(FilterOrder,:)).^2);


end

%========= Filtering input signal with the electrical Bessel filter =================
Elec_f = fft(Elec_input);
Filtered = fftshift(T_bessel).*Elec_f; %'ifftshift' inverses 'fftshift'

%====== Filtered Ekectrical Signal in time domain ============
y = real(ifft(Filtered));% Remove spurious imaginary part
