function Out = EyeAnalyzer_Relative(InSignal,info_bit,SampleTime)
%
% Usage: Out = EyeAnalyzer_Relative(InSignal,info_bit,SampleTime)
%
% Purpose = This module is to anaylzer an eye diagram with the synchronied pulse train.
%
% Input1 (InSignal) = Single channel signal (optical/electrical), Size = [1 x DataLength]
% Input2 (info_bit) = Bit sequence sent at the transmitter
%
% Output (Out) = [EyeClosure, EyeHeight,ExtinctionRatio, Q_value] 
% 
% Two figures will be generated.
%     1) figure1: Eye diagram with sampling instant
%     2) figire2: Histogram of sampled data
%
%==== Simulation Parameters ===============
%  SampleTime = 0.5 (default),  range = [0,1[, The normalized sampling instant relative to the bit frame.
%                      
% Note1) When the bit number is relatively small ( < 128), 'Synchronizer' may not work properly. 
%         In that case, this module will display "Eye is Closed" message errorously. By increasing bit numbers, this problem can be solved. 
% Note2) EyeClosure = (min(marks) - max(spaces))
%        EyeHeight = {(mean(marks)-3*std(marks)) - (mean(spaces)+3*std(spaces))}
%        ExtinctionRatio = (mean(marks)/mean(spaces))
%        Q_value = (|mean(marks)-mean(spaces)|/(std1+std0))
%
%------------------------------------------------------------
% $ Revision: 1.1 $ Date: 2011/02/22 by Jong-Hyung Lee  
%------------------------------------------------------------
%
%=====================================================================================
global samplesperbit Bits BitRate

if nargin<3, SampleTime = 0.5; end

s_type = ~isreal(InSignal);

if s_type == 1; %
    InSignal = InSignal.*conj(InSignal);
end

marks = mean(info_bit)*Bits; spaces = Bits - marks; % Number of Marks and Spaces in Bits

[Syn_out ref_pulse] = Synchronizer(InSignal,info_bit);

    Rel_samplesperbit = round(SampleTime*samplesperbit); 
    marks_v = Syn_out(ref_pulse==1);
    marks_c = marks_v(Rel_samplesperbit+samplesperbit*(0:(marks-1))); % Mark values at the SampleTime of the pulse train
    spaces_v = Syn_out(ref_pulse==0);
    spaces_c = spaces_v(Rel_samplesperbit+samplesperbit*(0:(spaces-1))); % Space values at the SampleTime of the pulse train

    min_mark = min(marks_c);  max_mark = max(marks_c);%  
    max_space = max(spaces_c); %

  %  To Check the eye quality 
 if min_mark < max_space
        EyePlotter2(InSignal,info_bit)
        disp('Eye is closed!  Will stop further calculations.')  
        disp('') 
        return
 end
%-------------------------------------------------------------------------                
                  
        ms1 = mean(marks_c); std1 = std(marks_c);
        ms0 = mean(spaces_c); std0 = std(spaces_c);   
        EyeClosure = min_mark - max_space; EyeHeight = (ms1-3*std1) - (ms0+3*std0);        
        ExtinctionRatio = ms1/ms0; Q_value = abs(ms1-ms0)/(std1+std0); 
        Out = [EyeClosure, EyeHeight, ExtinctionRatio, Q_value];
        
        fprintf('Eye Closure = %8.2e \n', EyeClosure);
        fprintf('Eye Height = %8.2e \n', EyeHeight);
        fprintf('Extinction Ratio = %6.2f \n', ExtinctionRatio);
        fprintf('Q value = %6.2f \n', Q_value);
         
        EyePlotter2(InSignal,info_bit)
        hold on
        t_SampleTime = SampleTime*(1e9/BitRate)*ones(1,20); %  EyePlotter2 plots eye in [ns].
        plot(t_SampleTime,1e3*linspace(0,max_mark*1.3,20),'k','LineWidth',2)
        text(t_SampleTime(1),1e3*max_mark*1.2,'Sampling Instant','FontSize',12)
       
       
      % To plot Histogram
             
          hist_data = [marks_c spaces_c];
          bin_number = min(50,length(hist_data));
          figure      
          hist(hist_data,bin_number); 
          h_data = hist(hist_data,bin_number);
          hmax = max(h_data); dh = hmax/10;
          xlabel('Magnitude of Sampled Data','FontSize',12),title('Histogram of Sampled Data of the Received Signal','FontSize',12)
          text(ms0+std0,hmax-dh,['mean0 = ' num2str(ms0)])
          text(ms0+std0,hmax-2*dh,['std0 = ' num2str(std0)])
          text(ms0+std0,hmax-4*dh,['mean1 = ' num2str(ms1)])
          text(ms0+std0,hmax-5*dh,['std1 = ' num2str(std1)])


