function OsnrdB = OSNR_Meter(Input_optical,BW_sig, BW_noise,NoiseFrequencyOffset,f_sig,MeasurementFilter_Type)
%
% Usage: OsnrdB = OSNR_Meter(Input_optical,BW_sig, BW_noise,NoiseFrequencyOffset,f_sig,MeasurementFilter_Type)
%
% Purpose = This module is to calcaulate optical signal-to-noise power ratio
%
%  Input Signal(Input_optical) = Optical Signal
%  Output(OsnrdB) = Optical signal-to-noise ratio[dB] measured at Ref_frequency
%
% ==== Physical Parameters ===== 
%   BW_sig = Signal Bandwidth [Hz], signal power will be calculated within BW_sig
%          = BitRate (default)
%   BW_noise = Noise Bandwidth [Hz], noise power will be calculated within BW_noise
%            = BitRate (default)
%   NoiseFrequencyOffset = Relative frequency from f_sig to  measure noise power 
%                        = f_sig + ChannelSpacing/2 (default)
%   f_sig = Signal center frequency to measure
%         = 0 (default)
%   MeasurementFilter_Type = 'Gauss','Rectangular'(ideal), 'Bessel'
%                          = 'Gauss' (default)  
%
%   Note1) Null-to-Null bandwidth of ideal NRZ signal = 2*BitRate ( BW_sig
%   should be increased when RZ signal is used.)
%   Note2) Noise power will be measured at +/- NoiseFrequencyOffset. In band noise power is estimated by the average of noise powers at the two points.  
%   Note3) BW_noise could be different from BW_sig. 
%
%------------------------------------------------------------
% $ Revision: 1.1 $ Date: 2011/02/22 by Jong-Hyung Lee  
%------------------------------------------------------------
%
%===============================================================================
global BitRate ChannelSpacing 

if nargin<6,
   MeasurementFilter_Type = 'Gauss';
end
if nargin<5,
   f_sig = 0;
end
if nargin<4
    NoiseFrequencyOffset= f_sig+ChannelSpacing/2;
end
if nargin<3,
 BW_noise = BitRate;
end
if nargin<2,
  BW_sig = BitRate;
end



[a,b] = size(Input_optical);

if a == 1  ; % single channel case
   Input_Optical_Fields = Input_optical;
else
   Input_Optical_Fields = sum(Input_optical);
end

 r_BW = BW_sig/BW_noise; % To consider the effect of the measurement BW difference between signl and noise.
 
if strcmp(MeasurementFilter_Type,'Rectangular') == 1

  s_n = OpticalFilter_Rectangular(Input_Optical_Fields,f_sig,BW_sig);
  n_p = OpticalFilter_Rectangular(Input_Optical_Fields,f_sig+NoiseFrequencyOffset,BW_noise);
  n_n = OpticalFilter_Rectangular(Input_Optical_Fields,f_sig-NoiseFrequencyOffset,BW_noise);

  s_nPower = PowerMeter(s_n); n_Power = .5*(PowerMeter(n_p) + PowerMeter(n_n));
  
  Osnr = s_nPower/(r_BW*n_Power) - 1;
  
  if Osnr < 0
      
      error('OSNR is too low. Check your configuration')
  else
      
     OsnrdB = 10*log10(Osnr);
  end   
  

elseif strcmp(MeasurementFilter_Type,'Gauss') == 1

  s_n = OpticalFilter_Gaussian(Input_Optical_Fields,f_sig,BW_sig);   
  n_p = OpticalFilter_Gaussian(Input_Optical_Fields,f_sig+NoiseFrequencyOffset,BW_noise);
  n_n = OpticalFilter_Gaussian(Input_Optical_Fields,f_sig-NoiseFrequencyOffset,BW_noise);

  s_nPower = PowerMeter(s_n); n_Power = .5*(PowerMeter(n_p) + PowerMeter(n_n));
 
  Osnr = s_nPower/(r_BW*n_Power) - 1;
  
  if Osnr < 0
      
      error('OSNR is too low. Check your configuration')
  else
      
     OsnrdB = 10*log10(Osnr);
  end   

  
elseif strcmp(MeasurementFilter_Type,'Bessel') == 1

  s_n = OpticalFilter_Bessel(Input_Optical_Fields,f_sig,BW_sig,3);  %Bessel Filter Order = 3
  n_p = OpticalFilter_Bessel(Input_Optical_Fields,f_sig+NoiseFrequencyOffset,BW_noise,3);
  n_n = OpticalFilter_Bessel(Input_Optical_Fields,f_sig-NoiseFrequencyOffset,BW_noise,3);

  s_nPower = PowerMeter(s_n); n_Power = .5*(PowerMeter(n_p) + PowerMeter(n_n));

 Osnr = s_nPower/(r_BW*n_Power) - 1;
  
  if Osnr < 0
      
      error('OSNR is too low. Check your configuration')
  else
      
     OsnrdB = 10*log10(Osnr);
  end   

  
else 
    
    error('Please check MeasurementFilter_Type.')
    
end
