function y = OpticalFilterInverted_Bessel(Optical_input,FilterCenter,BW3dB,FilterOrder) 
%
% Usage: y = OpticalFilterInverted_Bessel(Optical_input,FilterCenter,BW3dB,FilterOrder) 
%
% Purpose = This module represents an optical filter with inverted Bessel transfer characteristics. 
%
% Input Signal(Optical_input) = Optical Signal 
% Output Signal(y) = Optical Signal 
%
% === Physical Parameters === 
% FilterCenter = Deviation in [Hz] from the 'Ref_Frequency' [Hz]
%              = 0  (default)
% BW3dB = 3dB Bandwidth [Hz] 
%            = 4*BitRate (default)
% FilterOrder = positive integer [1,100]
%             = 3 (default)
% 
% (global variable) f_sym BitRate 
%
% Note1) Insertion Loss is NOT included. (IL can be modeled separately using function 'Attenuator'.)
% Note2) n_3dB = to normalize the 3dB bandwidth for FilterOrder >=3 
%               (at Cutoff Frequency, filter attenuation is greater than 3dB for FilterOrder >=3)
% Note3) FilterCenter = 0 corresponds to 'Ref_frequency'.  
%        Physical frequency = 'Ref_frequency' + 'f_sym'
% Note4) abs(FilterCenter) should be less than Fs/2, that is, less than max(f_sym)
%
%------------------------------------------------------------
% $ Revision: 1.1 $ Date: 2011/02/22 by Jong-Hyung Lee  
%------------------------------------------------------------
%
% Reference  1. Shammugan et al.,'Simulation of Communication Systems', Chapter 2, p69,  
%
%====================================================================================
global  f_sym BitRate

if nargin <4
    FilterOrder=3;   
end    
if nargin<3,
 BW3dB = 4*BitRate;
end
if nargin<2,
   FilterCenter = 0;
end

if FilterOrder < 1 || FilterOrder > 100
  error('FilterOrder is out of range: ');


elseif FilterOrder == 1 || FilterOrder == 2;

	p1 = 1i*2*(f_sym-FilterCenter)/BW3dB;   
	p1abs = abs(p1);  p1_center = find(p1abs == min(p1abs)); 

	B1(1,:) = 1+p1; B1(2,:) = 3 + 3*p1 + p1.*p1;
	T_bessel = B1(FilterOrder,p1_center(1))./B1(FilterOrder,:);
  
else

	n_3dB = sqrt((2*FilterOrder-1)*log(2));
	%n_3dB will make 3dB bandwidth approximately equal to parameter value 'BW3dB' for FilterOrder >= 3
	p1 = 1i*2*(f_sym-FilterCenter)/(BW3dB/n_3dB);
	p1abs = abs(p1); p1_center = find(p1abs == min(p1abs));     
    
	B1(1,:) = 1+p1; B1(2,:) = 3 + 3*p1 + p1.*p1;

	  for k = 3:FilterOrder
	
			B1(k,:) = (2*k-1)*B1(k-1,:) + p1.*p1.*B1(k-2,:);

  	  end

			T_bessel = B1(FilterOrder,p1_center(1))./B1(FilterOrder,:);

end

T_bessel_inverted = sqrt(1-T_bessel.*conj(T_bessel));

%========= Filtering input signal with the defined optical filter =================
E_out_f = fft(Optical_input);
Filtered = fftshift(T_bessel_inverted).*E_out_f;%'

%====== Filtered Optical Field in time domain
y = ifft(Filtered);

