function y = OpticalFilter_FabryPerot(Optical_input,FilterCenter,IL,BW3dB,FreeSpectrRange)
%
% Usage: y = OpticalFilter_FabryPerot(Optical_input,FilterCenter,IL,BW3dB,FreeSpectrRange)
%
% Purpose = This module represents a Fabry-Perot filter.
%
% Input Signal(Optical_input) = Optical Signal 
% Output Signal(y) = Optical Signal 
%
% ==== Physical Parameters ===== 
%   FilterCenter = Deviation in [Hz] from the 'Ref_Frequency'
%                = 0 (default)
%   IL = Insertion loss of the filter [dB]
%      = 3 (default)
%   BW3dB = 3dB badwidth of the filter [Hz]
%         = 4*BitRate (default)
%   FreeSpectrRange = Free Spectral range of the filter [Hz] 
%                   = 500e9[Hz] (default)
%   
% (global variables) f_sym BitRate
%
% Note1) FilterCenter = 0 corresponds to 'Ref_frequency'.  
%        Physical frequency = 'Ref_frequency' + 'f_sym'
% Note2) abs(FilterCenter) should be less than Fs/2, that is, less than max(f_sym).
%
%----------------------------------------------------------
% $ Revision: 1.1 $ Date: 2011/02/22 by Jong-Hyung Lee 
% $ Ref = OptiWave Component Manual
%------------------------------------------------------------
%
%====================================================================================
global f_sym BitRate

if nargin <5
    FreeSpectrRange =500e9; %[Hz]   
end    
if nargin<4,
    BW3dB = 4*BitRate;
end
if nargin<3
    IL = 3; %[dB]
end
if nargin<2,
   FilterCenter = 0;
end

%======== Define the filter transfer function ================
% Magnitude transfer function for optical field
phB_FSR = pi*BW3dB/FreeSpectrRange;
R = (2 + phB_FSR^2-sqrt((2+phB_FSR)^2-4))/2;

T_tranf =10^(-IL/10)*(1-R)./(1-R*exp(1i*2*pi*(f_sym-FilterCenter)/BW3dB));
%=====================================================================================

%========= Filtering input signal with the defined optical filter =================
E_out_f = fft(Optical_input);
Filtered = fftshift(T_tranf).*E_out_f;%

%====== Filtered Optical Field in time domain
y = ifft(Filtered);
