function [BER,i_th,Q] = QnBER_noISI_FixedSampleTime(InputElectrical,info_bit,SampleTime,...
                                    ThresholdType,Threshold,ThresholdSweep)
%
% Usage : [BER,i_th,Q] = QnBER_noISI_FixedSampleTime(InputElectrical,info_bit,SampleTime,...
%                                    ThresholdType,Threshold,ThresholdSweep)
%
% Purpose = This module is to calculate Q and BER based on Gaussian assumption. (without considering ISI effects)
%
% Input1(InputElectrical) = Single channel signal (photo-detected electrical signal), Size = [1 x DataLength]
% Input2 (info_bit) = Bit sequence sent at the transmitter
%
% Output1 (BER) = Estimated BER at the SampleTime
% Output2 (i_th) = Optimum physical threshold value when ThresholdType = 'Optimum' 
%                = 'Threshold' when ThresholdType = 'Absolute' (Meaningless case)
%                =  ms0 + 'Threshold'*(ms1-ms0) when ThresholdType = 'Relative'
% Output3 (Q) = Estimated Q-value at the SampleTime  
%
%==== Simulation Parameters ===============
%  SampleTime = 0.5 (default),  range = [0,1[, The normalized sampling instant relative to the bit frame.
%  ThresholdType = 'Optimum (default)','Absolute' or'Relative'  The method of threshold calculation.
%  Threshlold = 0.5 (default),  Parameter to determine threshold value 
%            Example) Threshold = 1e-3 if ThresholdType='Absolute',
%                     Threshold = 0.5 if ThresholdType='Relative'(i_th = m0 + 0.5*(m1-m0)), 
%            Note)    if ThresholdType = 'Optimum', Threhold value WILL NOT be used.
%                     Optimum BER (& Q) will be calculated by sweeping Threshold. 
%  ThresholdSweep =  'Yes' (Threshold vs BER (Threshold vs Q-value) will be plotted automatically.)
%                 = 'No' (Default, Optimum single values of BER (& Q), and i_th will be saved if ThresholdType = 'Optimum')
%             Note) 'Yes' will work only when ThresholdType = 'Optimum' 
%
%  Note1) Q-value is defined by the relationship, BER = (1/2)*erfc(Q/sqrt(2)) ~ exp(-Q^2/2)/(Q*sqrt(2*pi).

%----------------------------------------------------------
% $ Revision 1.0 $ Date: 2008/04/26 by Jong-Hyung Lee  
%   Revision 1.1 $ Date: 2008/10/24 by Jong-Hyung Lee
%          (New min BER = 1e-200   Old min BER = 1e-100)
%----------------------------------------------------------
% Reference 1. G.P. Agrawal, 'Fiber-optic Communication Systems', 2nd edition, Chapter 4
%
%======================================================================================================================
global samplesperbit Bits  

if nargin<6,   ThresholdSweep ='No'; end
if nargin<5,   Threshold = 0.5; end
if nargin<4,   ThresholdType = 'Optimum'; end
if nargin<3,   SampleTime = 0.5; end

marks = mean(info_bit)*Bits; spaces = Bits - marks; % Number of Marks and Spaces in Bits

%--------------------------------------------------------------------------
% 1. Generate the synchronized input pulse train
%--------------------------------------------------------------------------

[Syn_out ref_pulse] = Synchronizer(InputElectrical,info_bit);

%--------------------------------------------------------------------------
% 2. Calculate min Mark & max Space at the center of each pulses to see possible Eye closure
%    If min Mark < max Space, the eye is closed and stop further calculations
%--------------------------------------------------------------------------

Rel_samplesperbit = round(SampleTime*samplesperbit); 
marks_v = Syn_out(ref_pulse==1);
marks_c = marks_v(Rel_samplesperbit+samplesperbit*(0:(marks-1))); % Mark values at the SampleTime of the pulse train
spaces_v = Syn_out(ref_pulse==0);
% Considering PreSpaces & PostSpaces, Remove 3 bits at each ends. 
spaces_c = spaces_v(Rel_samplesperbit+samplesperbit*(4:(spaces-4))); % Space values at the SampleTime of the pulse train

min_mark = min(marks_c); %max_mark = max(marks_c);% 
max_space = max(spaces_c); %

      if min_mark < max_space

        EyePlotter(InputElectrical)
        disp('Eye closed!  Will stop further calculations.')
        disp('  ')
        disp('  ')
        return
        
      end 
       
%--------------------------------------------------------------------------
% 3. Calculate Mean and Standard Deviation of Sampled Data 
%--------------------------------------------------------------------------
         ms1 = mean(marks_c); std1 = std(marks_c);
         ms0 = mean(spaces_c); std0 = std(spaces_c);     
 
%--------------------------------------------------------------------------
% 4. Calculation of Q & BER  
%--------------------------------------------------------------------------

% -- 4.1 ThresholdType = 'Absolute', then i_th =  Threshold;

if strcmp(ThresholdType, 'Absolute') == 1
   
   i_th =  Threshold;
   Ber1 = .5*erfc(abs(ms1 - i_th)/sqrt(2)/std1);
   Ber0 = .5*erfc(abs(ms0 - i_th)/sqrt(2)/std0);
     
   BER = (marks*Ber1+(spaces-6)*Ber0)/(Bits-6);  % From Step #2, 6 Spaces has been removed.
   
                if BER < 1e-200
                     BER = 1e-200;
                end
                
   if nargout==3; % Q calculation
       
     log_ber = log(BER); % Note that it is not 'log10', but 'log' (natural logarithm)
     save ber_sim log_ber; % To be used in 'Q_value' function to calculate corresponding Q value.
   
        if log10(BER) > -9
            Q_ini = 6;           % initial assumption of Q value
        elseif log10(BER) > -50
            Q_ini = 15;
        else
            Q_ini = 20;
        end
   
     options = optimset('Display','off');% Suppress displaying calculation process by 'fzero'
     Q = fzero('Q_value',Q_ini,options); % 
     delete('ber_sim.mat');
   
   end
   
% -- 4.2 ThresholdType = 'Relative', then i_th = ms0 + Threshold*(ms1-ms0);

elseif strcmp(ThresholdType, 'Relative') == 1
   
   i_th = ms0 + Threshold*(ms1-ms0);  % 'Threshold' is the given relative value. 
   Ber1 = .5*erfc(abs(ms1 - i_th)/sqrt(2)/std1);
   Ber0 = .5*erfc(abs(ms0 - i_th)/sqrt(2)/std0);
     
   BER = (marks*Ber1+(spaces-6)*Ber0)/(Bits-6);  % From Step #2, 6 Spaces has been removed.
   
                if BER < 1e-200
                     BER = 1e-200;
                end   
                
    if nargout==3; % Q calculation
       
     log_ber = log(BER); % Note that it is not 'log10', but 'log' (natural logarithm)
     save ber_sim log_ber; % To be used in 'Q_value' function to calculate corresponding Q value.
   
        if log10(BER) > -9
            Q_ini = 6;           % initial assumption of Q value
        elseif log10(BER) > -50
            Q_ini = 15;
        else
            Q_ini = 20;
        end
   
     options = optimset('Display','off');% Suppress displaying calculation process by 'fzero'
     Q = fzero('Q_value',Q_ini,options); % 
     delete('ber_sim.mat');
   
    end
   
% -- 4.3 ThresholdType ='Optimum'. Threshold value is searched by first sweep(rough estimate) and then second sweep(fine sweep).   

elseif strcmp(ThresholdType, 'Optimum') == 1

    for k=1:11   ; % first threshold sweep(rough estimate)
   
        i_th(k) = ms0 + 0.1*(k-1)*(ms1-ms0); % 'i_th' sweeps from ms0 to ms1 (initial sweep)
        Ber1 = .5*erfc(abs(ms1 - i_th(k))/sqrt(2)/std1);
        Ber0 = .5*erfc(abs(ms0 - i_th(k))/sqrt(2)/std0);
     
        Ber(k) = (marks*Ber1+(spaces-6)*Ber0)/(Bits-6);  % From Step #2, 6 Spaces has been removed.
   
    end

    kb = median(find(Ber == min(Ber)));  % initial rough estimate of optimum value

    if kb ==1 || kb == 11
            disp('   ')
            disp('   ')
   	    	disp('Optimum threshold is out of simulation range!')
            disp('   ')
            return
    end

    i_th_sw = (ms0 + 0.1*(kb-2)*(ms1-ms0)):0.01*(ms1-ms0):(ms0 + 0.1*kb*(ms1-ms0)); % 21 step sweep
  
        for kk = 1:length(i_th_sw);  % 2nd sweep, fine sweep to find optimum values
    
            Ber1 = .5*erfc(abs(ms1 - i_th_sw(kk))/sqrt(2)/std1);
            Ber0 = .5*erfc(abs(ms0 - i_th_sw(kk))/sqrt(2)/std0);
            Ber(kk) = (marks*Ber1+(spaces-6)*Ber0)/(Bits-6); % From Step #2, 6 Spaces has been removed.
             
            if Ber(kk) < 1e-200
                Ber(kk) = 1e-200;
            end           
   
        end    
        

       
        %  When ThresholdType ='Optimum', 
        
        if strcmp(ThresholdSweep, 'No') == 1
       
          kb2 = Ber == min(Ber);
          BER = min(Ber); i_th = i_th_sw(kb2);
       
          if nargout==3; % Q calculation
       
                log_ber = log(BER); % Note that it is not 'log10', but 'log' (natural logarithm)
                save ber_sim log_ber; % To be used in 'Q_value' function to calculate corresponding Q value.
   
                    if log10(BER) > -9
                        Q_ini = 6;           % initial assumption of Q value
                    elseif log10(BER) > -50
                        Q_ini = 15;
                    else
                        Q_ini = 20;
                    end
   
                options = optimset('Display','off');% Suppress displaying calculation process by 'fzero'
                Q = fzero('Q_value',Q_ini,options); % 
                delete('ber_sim.mat');
   
          end
            
        elseif strcmp(ThresholdSweep, 'Yes') == 1
       
          BER = Ber; i_th = i_th_sw;
        
           if nargout==3; % Q calculation
                
               BB = length(Ber);
               
               for kk=1:BB
                   
                log_ber = log(Ber(kk)); % Note that it is not 'log10', but 'log' (natural logarithm)
                save ber_sim log_ber; % To be used in 'Q_value' function to calculate corresponding Q value.
   
                    if log10(Ber(kk)) > -9
                        Q_ini = 6;           % initial assumption of Q value
                    elseif log10(Ber(kk)) > -50
                        Q_ini = 15;
                    else
                        Q_ini = 20;
                    end
   
                options = optimset('Display','off');% Suppress displaying calculation process by 'fzero'
                Q(kk) = fzero('Q_value',Q_ini,options); % 
                delete('ber_sim.mat');
   
               end
           
           figure      
           subplot 211
           semilogy(i_th,BER,'o-');
           grid on
           xlabel('Threshold','FontSize',12);ylabel('BER','FontSize',12)
           title('Threshold vs. BER','FontSize',12)
           subplot 212
           plot(i_th,Q,'*-');
           xlabel('Threshold','FontSize',12);ylabel('Q','FontSize',12)
           title('Threshold vs. Q','FontSize',12)
           grid on
           
           else
               
           figure      
           semilogy(i_th,BER,'o-');
           grid on
           xlabel('Threshold','FontSize',12);ylabel('BER','FontSize',12)
           title('Threshold vs. BER','FontSize',12)
               
               
           end
           
        end
        
 end
        
 
