function [yt yf] = SingleModeFiber(OpticalFieldIn,Fiber_Length,FiberAttenuation,Dispersion,DispersionSlope,...
                             NonlinearIndex,CoreArea,MaxPhaseChange)
%
% Usage: [yt yf] = SingleModeFiber(OpticalFieldIn,Fiber_Length,FiberAttenuation,Dispersion,DispersionSlope,...
%                             NonlinearIndex,CoreArea,MaxPhaseChange)
%
% Purpose = To solve the nonlinear Schrodinger equation using the Split-Step Fourier method with equally spaced simulation step, dz. 
%                    (Symmetrical simulation: '1/2dispersion --> Nonlinear-->1/2dispersion')
%
% Included physical effects of a single mode fiber = Fiber Attenuation, First order GVD, 
%                                                    Second-order GVD, Kerr nonlinear effects(SPM,CPM,FWM)                         
%
% Input(OpticalFieldIn) = Optical signal (time domain)
% Output1(yt) = Optical signal (time domain)
% Output2(yf) = Optical signal (frquency domain)
%
%=== Physical Parameters with Typical values of SMF-28 =========
%  Fiber_Length = 50e3; %[m]
%  FiberAttenuation = 0.2e-3; %[dB/m]
%  Dispersion = 16e-6;   %[s/m^2]
%  DispersionSlope = 0.08e3;   %[s/m^3]
%  NonlinearIndex = 2.6e-20; %[m^2/W]
%  CoreArea = 80e-12; %[m^2]
%
%=== Simulation Parameters =====
%  MaxPhaseChange = 0.005; %[rad] Maximum allowable phase change by one nonlinear operation step
%
%  (global variables) Ref_Frequency  f_sym  [Hz];
%
% Note1) M steps of '1/2dispersion --> Nonlinear-->1/2dispersion'
%        = '1/2dispersion'(first step) + (M-1) steps of 'Nonlinear--> Dispersion' + 'Nonlinear'(last) + '1/2dispersion'(last)  
% Note2) If the fiber nonlinearity is absent, use 'SingleModeFiber_Lin'.
%
%------------------------------------------------------------
% $ Revision: 1.1 $ Date: 2011/02/22 by Jong-Hyung Lee  
%------------------------------------------------------------
% 
% Reference 1. G.P. Agrawal, 'Nonlinear Fiber Optics', 4th edition, Chapter 2 & Appendix B
%
%=====================================================================================
global Ref_Frequency f_sym

if nargin <8
    MaxPhaseChange = 0.005; %[rad]
end
if nargin <7
    CoreArea = 80e-12; %[m^2]
end
if nargin <6
   NonlinearIndex = 2.6e-20; %[m^2/W]
end
if nargin <5
    DispersionSlope=0.08e3;   %[s/m^3]   
end    
if nargin<4,
    Dispersion = 16e-6;   %[s/m^2]
end
if nargin<3,
   FiberAttenuation = 0.2e-3; %[dB/m]
end
if nargin<2,
   Fiber_Length = 50e3; %[m]
end
%==========================================================================
c_light = 2.99793e8;       %[m/sec] speed of light
gamma = 2*pi*NonlinearIndex*Ref_Frequency/(c_light*CoreArea); %[1/(W meter)]
alpha = FiberAttenuation/(10*log10(exp(1))); %[1/m] Power Loss in linear scale

if gamma == 0; % Nonlinearty is absent.
    
    [yt yf] = SingleModeFiber_Lin(OpticalFieldIn,Fiber_Length,FiberAttenuation,Dispersion,DispersionSlope);

else
    
InputAvePower = PowerMeter(OpticalFieldIn); %Measure power level of input optical signal
dz1 = MaxPhaseChange/(gamma*InputAvePower); 
step_num = ceil(Fiber_Length/dz1); % make to the nearest integers towards infinity.  
% Define new simulation step length to make 'step_num' an integer.
% 'dz' is always smaller than 'dz1', so simulation phase change due to nonlinearity is smaller than 'MaxPhaseChange'.
dz = Fiber_Length/step_num;  

w_range = 2*pi*(Ref_Frequency + f_sym); %physical optical frequency
w_base = 2*pi*f_sym; %base band optical frequency

beta2 = -2*pi*c_light*Dispersion./(w_range.^2);
beta3 = (2*pi*c_light)^2*DispersionSlope./(w_range.^4) + 4*pi*c_light*Dispersion./(w_range.^3);

%------ Dispersion & Nonlinear Phase Factor with step size = dz
% Dis_ope = frequency domain, Non_ope = time domain

lin = fftshift(1i*beta2.*w_base.^2/2 + 1i*beta3.*w_base.^3/6 - alpha/2);%(d/dt)=(-iw) 
Dis_ope = exp(lin*dz);  
Dis_ope2 = exp(lin*dz/2);  % for the first & the last step 

Non_ope = 1i*gamma*dz;   

%=======================================================================================================
in_f = fft(OpticalFieldIn);

% Initial Dispersion alone with dz/2 step (Symetrical Algorithm)
opt_f = in_f.*Dis_ope2;
opt_t = ifft(opt_f);

% Propagation through fiber from dz/2 to dz*(step_num-1)
for n=1:(step_num-1);  % 
    
    t_temp = opt_t.* exp(opt_t.*conj(opt_t).*Non_ope); % Nonlinearity alone with dz step
    f_temp = fft(t_temp).*Dis_ope;     % Dispersion alone with dz step
    opt_t = ifft(f_temp);            % Back to time domain signal after dz propagation
end

t_temp = opt_t.* exp(opt_t.*conj(opt_t).*Non_ope); % Last nonlinear step with dz 
out_f = fft(t_temp);

yf = out_f.*Dis_ope2;% Last linear step with dz/2 
yt = ifft(yf);

end
