function [y,info_bits] = TxModAmplitude_NRZ(UnmodulatedPower,Linewidth,ModulationIndex,...
                         PreSpaces,PostSpaces,RiseTime)
%
% Usage: [y,info_bits] = TxModAmplitude_NRZ(UnmodulatedPower,Linewidth,ModulationIndex,...
%                        PreSpaces,PostSpaces,RiseTime)
%
% Purpose = This module is to model an externally modulated laser by
% ModulatorAmplitude using NRZ line coding.
%          
% TxModAmplitude_NRZ = info_bits(PRBS) + NRZCoder + RiseTimeAdjust
%                              + LaserCW + ModulatorAmplitude
%
% Input = Not required.
% Output1(y) = Optical modulated signal 
% Output2(info_bits) = info_bits for BER calculation at the receiver.
%
%=== Physical Parameters with Default Values=======
% UnmodulatedPower= 1e-3 (default);  % [W] CW Laser 
% Linewidth= 10e6 (default);     %[Hz] CW Laser
%
% ModulationIndex = 0.9 (default); %range [0,1] (Extinction ratio control) ModulatorAmplitude 
% ChirpFactor = 0 (default); %range ]-inf,inf[ ModulatorAmplitude
%
% PreSpaces = 2 (default); % number of leading zero bits, infobits_PRBS
% PostSpaces = 2 (default); % number of trailing zero bits, infobits_PRBS
%
% RiseTime = 1/16/BitRate (default);% RiseTimeAdjust  [sec]
%
% Note) Extinction Ratio [dB] = -10log10(1-ModulationIndex)
%
%------------------------------------------------------------
% $ Revision: 1.0 $ Date: 2011/02/22 by Jong-Hyung Lee  
%------------------------------------------------------------
%
%====================================================================================
global BitRate

if nargin <6
    RiseTime = 1/16/BitRate;
end
if nargin <5
    PostSpaces = 2;   
end    
if nargin<4,
    PreSpaces = 2;
end
if nargin<3,
   ModulationIndex = 0.9;
end
if nargin<2,
   Linewidth= 10e6;
end
if nargin==0,
    UnmodulatedPower=1e-3;
end

%===================================================================
info_bits = infobits_PRBS(PreSpaces,PostSpaces); 
ElectricalPreshaped = NRZCoder(info_bits);
Elec_mod = RiseTimeAdjust(ElectricalPreshaped,RiseTime); 
%========================================================================
% CW Optical Signal
E_CW = LaserCW(UnmodulatedPower,Linewidth);
%========================================================================

y = ModulatorAmplitude(E_CW,Elec_mod,ModulationIndex);

